"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const single_line_log_1 = require("single-line-log");
const https = require('follow-redirects/https');
const checkSource = (sourcePath) => __awaiter(this, void 0, void 0, function* () {
    let symbolFiles;
    const symbolPath = path.join(sourcePath, 'wrapper', 'symbols');
    const constantsFile = path.join(sourcePath, 'wrapper', 'constants.json');
    const packageFile = path.join(sourcePath, 'package.json');
    try {
        symbolFiles = yield fs.readdir(symbolPath);
    }
    catch (error) {
        throw new Error(`Could not find symbols in ${symbolPath} from downloaded ZIP file.`);
    }
    try {
        yield fs.readFile(constantsFile);
    }
    catch (error) {
        throw new Error(`Could not find constants file ${constantsFile} in downloaded ZIP file.`);
    }
    if (!symbolFiles.some(fileName => /.*json/.test(fileName))) {
        throw new Error(`Could not find wrapper files in ${sourcePath}.`);
    }
    try {
        const packageData = yield fs.readFile(packageFile, 'utf8');
        const jsonData = JSON.parse(packageData);
        return jsonData.version;
    }
    catch (error) {
        throw new Error(`Error reading libsodium package file: ${error}`);
    }
});
exports.checkSource = checkSource;
const compareVersionNumbers = (version1, version2) => {
    var v1parts = version1.split('.');
    var v2parts = version2.split('.');
    const isPositiveInteger = (x) => /^\d+$/.test(x);
    const validateParts = (parts) => {
        for (var i = 0; i < parts.length; ++i) {
            if (!isPositiveInteger(parts[i])) {
                return false;
            }
        }
        return true;
    };
    if (!validateParts(v1parts) || !validateParts(v2parts)) {
        return NaN;
    }
    for (var i = 0; i < v1parts.length; ++i) {
        if (v2parts.length === i) {
            return 1;
        }
        if (v1parts[i] === v2parts[i]) {
            continue;
        }
        if (v1parts[i] > v2parts[i]) {
            return 1;
        }
        return -1;
    }
    if (v1parts.length != v2parts.length) {
        return -1;
    }
    return 0;
};
exports.compareVersionNumbers = compareVersionNumbers;
const httpsGetFileAsync = (url, fileName, showProgress = false) => {
    const MEGABYTE = 0x100000;
    const KILOBYTE = 0x400;
    let startedAt;
    return new Promise((resolve, reject) => {
        https
            .get(url.href, (res) => {
            const { statusCode } = res;
            if (statusCode !== 200) {
                return reject(new Error(`Request to ${url} failed.\nStatus code: ${statusCode}`));
            }
            const file = fs.createWriteStream(fileName);
            const lengthRaw = res.headers['content-length'];
            const usesChunkedEncoding = !lengthRaw;
            const length = usesChunkedEncoding ? 0 : parseInt(lengthRaw, 10);
            let total = usesChunkedEncoding ? 0 : length / MEGABYTE;
            let transferred = 0;
            let elapsed = 0;
            res
                .on('data', chunk => {
                transferred += chunk.length;
                if (usesChunkedEncoding) {
                    total = transferred / MEGABYTE;
                }
                elapsed = (Date.now() - startedAt) / 1000;
                const percent = ((100.0 * transferred) / length).toFixed(2);
                const speed = transferred / elapsed;
                const speedFormatted = speed < MEGABYTE
                    ? `${~~(speed / KILOBYTE)} kB`
                    : `${(speed / MEGABYTE).toFixed(2)} MB`;
                single_line_log_1.stdout(`${usesChunkedEncoding ? '?' : percent + ' %'} of ${total.toFixed(2)} MB (${speedFormatted}/s)`);
            })
                .on('end', () => {
                const seconds = ~~elapsed;
                console.log(` in ${seconds} second${seconds === 1 ? '' : 's'}.`);
                resolve(fileName);
            })
                .pipe(file);
        })
            .on('response', () => (startedAt = Date.now()))
            .on('error', (error) => reject(`Could not download libsodium.js: ${error.message}`));
    });
};
exports.httpsGetFileAsync = httpsGetFileAsync;
